/*
 *   Karel v Ccku
 *
 *   Copyright (c) 2012 Tomas Blaha
 *
 *   Permission is hereby granted, free of charge, to any person
 *   obtaining a copy of this software and associated documentation
 *   files (the "Software"), to deal in the Software without
 *   restriction, including without limitation the rights to use,
 *   copy, modify, merge, publish, distribute, sublicense, and/or sell
 *   copies of the Software, and to permit persons to whom the
 *   Software is furnished to do so, subject to the following
 *   conditions:
 *
 *   The above copyright notice and this permission notice shall be
 *   included in all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *   OTHER DEALINGS IN THE SOFTWARE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>

#include <SDL/SDL.h>
#include <SDL_image.h>

#include "karel.h"

extern World world;
int karel;

float nahoda(int minimum, int maximum)
{
	return float (rand()) / RAND_MAX * (maximum - minimum + 1) + minimum;
}

void ViewUpdateAt(unsigned int x, unsigned int y)
{
	SDL_Rect src, dest;
	Robot *it;

	src.y = 0;
	src.w = src.h = world.grid;
	dest.x = x * world.grid + world.view_offset_x;
	dest.y = y * world.grid + world.view_offset_y;
	if(world.map[y * world.width + x].wall)
	{
		src.x = 5 * world.grid;
		if(SDL_BlitSurface(world.data, &src, world.screen, &dest) < 0)
		{
			fprintf(stderr, "SDL_BlitSurface() failed: %s\n", SDL_GetError());
			exit(-1);
		}
	}
	else
	{
		src.x = 6 * world.grid;
		if(SDL_BlitSurface (world.data, &src, world.screen, &dest) < 0)
		{
			fprintf(stderr, "SDL_BlitSurface() failed: %s\n", SDL_GetError());
			exit(-1);
		}
	}
	// Items
	if(world.map[y * world.width + x].item != -1)
	{
		src.x = world.map[y * world.width + x].item * world.grid;
		src.y = 0;
		src.w = src.h = world.grid;
		dest.x = x * world.grid + world.view_offset_x;
		dest.y = y * world.grid + world.view_offset_y;
		if(SDL_BlitSurface(world.items, &src, world.screen, &dest) < 0)
		{
			fprintf(stderr, "SDL_BlitSurface() failed: %s\n", SDL_GetError());
			exit(-1);
		}
	}
	// Marks
	if(world.map[y * world.width + x].num_marks)
	{
		src.x = 512 + (world.map[y * world.width + x].num_marks - 1) * world.grid;
		src.y = 0;
		src.w = src.h = world.grid;
		dest.x = x * world.grid + world.view_offset_x;
		dest.y = y * world.grid + world.view_offset_y;
		if(SDL_BlitSurface(world.data, &src, world.screen, &dest) < 0)
		{
			fprintf(stderr, "SDL_BlitSurface() failed: %s\n", SDL_GetError());
			exit(-1);
		}
	}
	// Robots
	src.y = 0;
	src.w = src.h = world.grid;
	for(it = world.robots; it; it = it->next)
	{
		if((it->x == x) && (it->y == y))
		{
			src.x = it->dir * world.grid;
			dest.x = x * world.grid + world.view_offset_x;
			dest.y = y * world.grid + world.view_offset_y;
			if(SDL_BlitSurface(world.data, &src, world.screen, &dest) < 0)
			{
				fprintf(stderr, "SDL_BlitSurface() failed: %s\n", SDL_GetError());
				exit(-1);
			}
		}
	}
	SDL_UpdateRect(world.screen, x * world.grid + world.view_offset_x, y * world.grid + world.view_offset_y, world.grid, world.grid);
}

void ViewUpdate()
{
	int i, j;

	for(i = 0; i < world.width; i++)
		for(j = 0; j < world.height; j++)
		{
			ViewUpdateAt(i, j);
		}
}

void ViewDrawCursor(unsigned int x, unsigned int y)
{
	SDL_Rect src, dest;

	src.x = 7 * world.grid;
	src.y = 0;
	src.w = src.h = world.grid;
	dest.x = x * world.grid + world.view_offset_x;
	dest.y = y * world.grid + world.view_offset_y;

	if(SDL_BlitSurface(world.data, &src, world.screen, &dest) < 0)
	{
		fprintf(stderr, "SDL_BlitSurface() failed: %s\n", SDL_GetError());
		exit(-1);
	}
	SDL_UpdateRect(world.screen, x * world.grid + world.view_offset_x, y * world.grid + world.view_offset_y, world.grid, world.grid);
}

// 0 nic, -2 preruseni vykonavani, -3 ukonceni prostredi
int ViewPollEvents()
{
	SDL_Event event;

	while(SDL_PollEvent(&event))
	{
		switch(event.type)
		{
		case SDL_QUIT:
			exit(1);
		case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
			case SDLK_ESCAPE:
				exit(1);
			}
		case SDL_KEYUP:
			switch(event.key.keysym.sym)
			{
			case SDLK_SPACE:
				if(event.key.state == SDL_PRESSED) world.fast = 1;
				else world.fast = 0;
				break;
			}
			break;
		}
	}
	return 0;
}

int ViewInit()
{
	if(SDL_Init(SDL_INIT_VIDEO) < 0)
	{
		fprintf(stderr, "SDL_Init() failed: %s\n", SDL_GetError());
		return -1;
	}
	if(!(world.screen = SDL_SetVideoMode(world.width * world.grid, world.height * world.grid, 0, 0)))
	{
		fprintf(stderr, "SDL_SetVideoMode() failed: %s\n", SDL_GetError());
		return -1;
	}

	FILE *f;
	int i, j;
	char str[32];

	if(!(world.data = IMG_Load("data/data.png")))
	{
		fprintf(stderr, "IMG_Load() failed\n");
		return -1;
	}
	if(!(world.items = IMG_Load("data/items.png")))
	{
		fprintf(stderr, "IMG_Load() failed\n");
		return -1;
	}

	f = fopen("data/items.desc", "rt");
	if(!f)
	{
		exit(0);
	}
	i = 0;
	while(!feof(f))
	{
		fgets(str, 32, f);
		for(j = 0; j < strlen(str); j++)
			if(str[j] == '\n')
			{
				str[j] = 0;
				break;
			}
		strcpy(world.item_names[i++], str);
	}
	fclose(f);
	world.num_items = i;

	ViewUpdate();
	return 0;
}

World world;
int world_inited = 0;

Robot *GetRobot(int handle = -1)
{
	Robot *it;

	if(handle == -1) handle = karel;

	if((world_inited == 2) && (it = world.robots))
	{
		do
		{
			if(it->handle == handle) return it;
		}
		while(it = it->next);
	}
	return NULL;
}

int KInit()
{
	struct timeval tv;
	gettimeofday(&tv, NULL);
	srand(tv.tv_usec);

	if(world_inited == 0)
	{
		world_inited = 1;

		world.grid = 64;
		world.robots = NULL;
		world.autoincrement = 1;
		world.fast = 0;
		world.view_offset_x = 0;
		world.view_offset_y = 0;
		world.command_delay = 250;
	}
	else
	{
		fprintf(stderr, "Library already inited.");
		return -1;
	}
}

int KSaveWorld(const char *fname)
{
	FILE *f;

	if(!(f = fopen(fname, "wb"))) return -1;
	fwrite(&world.width, sizeof(int), 1, f);
	fwrite(&world.height, sizeof(int), 1, f);
	fwrite(world.map, sizeof(WorldField), world.width * world.height, f);
	fclose(f);
	return 1;
}

int KLoadWorld(const char *fname)
{
	FILE *f;
	int width, height;

	if(!world_inited) return -1;
	if(!(f = fopen(fname, "rb"))) return 0;
	fread(&width, sizeof(int), 1, f);
	fread(&height, sizeof(int), 1, f);
	if(world_inited == 1) KCreateWorld(width, height);
	if((width != world.width) || (height != world.height))
	{
		fclose(f);
		return 0;
	}
	fread(world.map, sizeof(WorldField), world.width * world.height, f);
	fclose(f);
	ViewUpdate();
	SDL_Delay(1000);
	return 1;
}

int KCreateWorld(unsigned int width, unsigned int height)
{
	int i;

	if(world_inited == 1)
	{
		world_inited = 2;

		world.width = width;
		world.height = height;

		world.map = (WorldField*)malloc(sizeof(WorldField) * world.width * world.height);
		for(i = 0; i < world.width * world.height; i++)
		{
			world.map[i].wall = 0;
			world.map[i].item = -1;
			world.map[i].num_marks = 0;
		}

		if(ViewInit())
		{
			fprintf(stderr, "View initialisation failed.\n");
			return -1;
		}
		return 0;
	}
	else
	{
		fprintf(stderr, "World already created or library not inited.");
		return -1;
	}
}


int konec()
{
	cekej();
	if(world_inited == 2)
	{
		SDL_Quit();
		free(world.map);
		return 0;
	}
	return -1;
}

int KCreateRobot()
{
	return KCreateRobotAt(0, world.height - 1, 0);
}

int KCreateRobotAt(unsigned int x, unsigned int y, unsigned int dir)
{
	Robot *it;

	if(world_inited != 2)
	{
		fprintf(stderr, "World not created.");
		return -1;
	}

	if(it = world.robots)
	{
		do
		{
			if((it->x == x) && (it->y == y)) return 0;
		}
		while(it = it->next);
	}

	if(world.robots == NULL)
	{
		if(!(world.robots = (Robot*)malloc(sizeof(Robot))))
		{
			fprintf(stderr, "Memory allocation failed.");
			exit(-1);
		}
		it = world.robots;
	}
	else
	{
		it = world.robots;
		while(it->next != NULL) it = it->next;
		if(!(it->next = (Robot*)malloc(sizeof(Robot))))
		{
			fprintf(stderr, "Memory allocation failed.");
			exit(-1);
		}
		it = it->next;
	}
	it->next = NULL;
	it->handle = ++world.autoincrement;
	if(x < world.width) it->x = x;
	else
	{
		fprintf(stderr, "Placement out of bounds.");
		exit(-1);
	}
	if(y < world.height) it->y = y;
	else
	{
		fprintf(stderr, "Placement out of bounds.");
		exit(-1);
	}
	if(dir < 4) it->dir = dir;
	else
	{
		fprintf(stderr, "Senseless direction.");
		exit(-1);
	}
	ViewUpdateAt(x, y);
	return world.autoincrement;
}

void priprav(const int w, const int h)
{
	KInit();
	KCreateWorld(w, h);
	karel = KCreateRobot();
}

void razChodbu(int w, int h, int x, int y)
{
	int x2, y2;
	world.map[y * w + x].wall = 0;

	for(;;)
	{
		int s = nahoda(0, 3);
		int j;
		for(j = 0; j < 4; j++)
		{
			x2 = x; y2 = y;
			switch(s)
			{
			case 0:
				x2 = x + 1;
				break;
			case 1:
				y2 = y - 1;
				break;
			case 2:
				x2 = x - 1;
				break;
			case 3:
				y2 = y + 1;
				break;
			}

			if(x2 >= 0 && x2 < w && y2 >= 0 && y2 < h)
			{
				int status = 0;
				switch(s)
				{
				case 0:
					if((y2 - 1 < 0 || world.map[(y2 - 1) * w + x2].wall)
							&& (y2 + 1 >= h || world.map[(y2 + 1) * w + x2].wall)
							&& (x2 + 1 >= w || world.map[y2 * w + x2 + 1].wall))
						status = 1;
					break;
				case 1:
					if((x2 - 1 < 0 || world.map[y2 * w + x2 - 1].wall)
							&& (x2 + 1 >= w || world.map[y2 * w + x2 + 1].wall)
							&& (y2 - 1 < 0 || world.map[(y2 - 1) * w + x2].wall))
						status = 1;
					break;
				case 2:
					if((y2 - 1 < 0 || world.map[(y2 - 1) * w + x2].wall)
							&& (y2 + 1 >= h || world.map[(y2 + 1) * w + x2].wall)
							&& (x2 - 1 < 0 || world.map[y2 * w + x2 - 1].wall))
						status = 1;
					break;
				case 3:
					if((x2 - 1 < 0 || world.map[y2 * w + x2 - 1].wall)
							&& (x2 + 1 >= w || world.map[y2 * w + x2 + 1].wall)
							&& (y2 + 1 >= h || world.map[(y2 + 1) * w + x2].wall))
						status = 1;
					break;
				}


				if(status) break;
			}
			s = (s + 1) % 4;
		}
		if(j < 4)
		{
			x = x2; y = y2;
			world.map[y * w + x].wall = 0;
		}
		else break;
	}
}

void priprav(const char* uloha)
{
	KInit();

	if(!strcmp(uloha, "ruzne dvorky"))
	{
		KCreateWorld(nahoda(4, 12), nahoda(4, 12));
		karel = KCreateRobot();
	}
	if(!strcmp(uloha, "brana"))
	{
		KCreateWorld(9, 9);
		for(int i = 0; i < 9; i++)
			world.map[36 + i].wall = i != 4;

		karel = KCreateRobot();

		polozNa(8, 0);

		ViewUpdate();

	}
	if(!strcmp(uloha, "schod"))
	{
		KCreateWorld(12, 4);
		for(int i = 0; i < 9; i++)
		world.map[39 + i].wall = 1;
		
		karel = KCreateRobot();
		ViewUpdate();
	}
	if(!strcmp(uloha, "bludiste"))
	{
		int w = 12, h = 12;
		KCreateWorld(12, 12);
		
		for(int i = 0; i < w * h; i++)
			world.map[i].wall = 1;
		
		razChodbu(w, h, 0, 11);
		for(int i = 0; i < 100; i++)
		{
//			SDL_Delay(2000);
			int x = nahoda(0, 11);
			int y = nahoda(0, 11);
			if(world.map[y * w + x].wall == 0)
			{
				razChodbu(w, h, x, y);
				ViewUpdate();
			}
		}
		for(int i = 0; i < w * h; i++)
		{
			if(world.map[i].wall == 0)
			{
				world.map[i].num_marks++;
				break;
			}
		}

		karel = KCreateRobot();
		ViewUpdate();
	}

}

int vlevo(int handle)
{
	Robot *robot;
	int stat;

	if(robot = GetRobot(handle))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;

		robot->dir = (robot->dir + 1) & 3;

		ViewUpdateAt(robot->x, robot->y);
		return 1;
	}
	chyba();
}

int vpravo(int handle)
{
	Robot *robot;
	int stat;

	if(robot = GetRobot(handle))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;

		robot->dir = (robot->dir - 1) & 3;

		ViewUpdateAt(robot->x, robot->y);
		return 1;
	}
	chyba();
}

int presunNa(unsigned int x, unsigned int y, int handle = -1)
{
	Robot *robot, *it;
	int ox, oy;
	int stat;

	if(robot = GetRobot(handle))
	{
		if((x >= 0) && (y >=0) && (x < world.width) && (y < world.height))
		{
			if(!world.fast) SDL_Delay(world.command_delay);
			if(stat = ViewPollEvents()) return stat;

			if(world.map[y * world.width + x].wall) return 0;

			it = world.robots;
			do
			{
				if((it->x == x) && (it->y == y)) return 0;
			}
			while(it = it->next);

			ox = robot->x;
			oy = robot->y;
			robot->x = x;
			robot->y = y;

			ViewUpdateAt(ox, oy);
			ViewUpdateAt(robot->x, robot->y);
			return 1;
		}
		chyba();
		return 0;
	}
	chyba();
}

int krok(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		switch(robot->dir)
		{
		case 0:
			return presunNa(robot->x + 1, robot->y, handle);
		case 1:
			return presunNa(robot->x, robot->y - 1, handle);
		case 2:
			return presunNa(robot->x - 1, robot->y, handle);
		case 3:
			return presunNa(robot->x, robot->y + 1, handle);
		}
	}
	chyba();
}

int jeZedNa(unsigned int x, unsigned int y, int handle)
{
	int stat;

	if(GetRobot(handle) && (x >= 0) && (y >= 0) && (x < world.width) && (y < world.height))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;
		if(world.map[y * world.width + x].wall) return 1;
		else return 0;
	}
	return 1;
}

int jeZed(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		switch(robot->dir)
		{
		case 0:
			return jeZedNa(robot->x + 1, robot->y, handle);
		case 1:
			return jeZedNa(robot->x, robot->y - 1, handle);
		case 2:
			return jeZedNa(robot->x - 1, robot->y, handle);
		case 3:
			return jeZedNa(robot->x, robot->y + 1, handle);
		}
	}
	chyba();
}

int jePredmetNa(unsigned int x, unsigned int y, int handle)
{
	int stat;

	if(GetRobot(handle) && (x >= 0) && (y >=0) && (x < world.width) && (y < world.height))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;

		if(world.map[y * world.width + x].item != -1) return 1;
		else return 0;
	}
	return 0;
}

int jePredmet(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return jePredmetNa(robot->x, robot->y, handle);
	}
	chyba();
}

int jeZnackaNa(unsigned int x, unsigned int y, int handle)
{
	int stat;

	if(GetRobot(handle) && (x >= 0) && (y >= 0) && (x < world.width) && (y < world.height))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;

		if(world.map[y * world.width + x].num_marks) return 1;
		else return 0;
	}
	return 0;
}

int jeZnacka(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return jeZnackaNa(robot->x, robot->y, handle);
	}
	chyba();
}

int jeRobotNa(unsigned int x, unsigned int y, int handle)
{
	Robot *it;
	int stat;

	if(GetRobot(handle) && (x >= 0) && (y >=0) && (x < world.width) && (y < world.height))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;

		for(it = world.robots; it != NULL; it = it->next)
		{
			if((it->x == x) && (it->y == y)) return 1;
		}
		return 0;
	}
	return 0;
}

int jeRobot(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		switch(robot->dir)
		{
		case 0:
			return jeRobotNa(robot->x + 1, robot->y, handle);
		case 1:
			return jeRobotNa(robot->x, robot->y - 1, handle);
		case 2:
			return jeRobotNa(robot->x - 1, robot->y, handle);
		case 3:
			return jeRobotNa(robot->x, robot->y + 1, handle);
		}
	}
	chyba();
}

int polozPredmetNa(const char *item, unsigned int x, unsigned int y, int handle)
{
	int i,stat;

	if(GetRobot(handle) && (x >= 0) && (y >=0) && (x < world.width) && (y < world.height))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;
		if(world.map[y * world.width + x].item != -1) chyba();
		
		for(i = 0; i < world.num_items; i++)
		{
			if(strcmp(world.item_names[i], item) == 0)
			{
				world.map[y * world.width + x].item = i;
				ViewUpdateAt(x, y);
				return 1;
			}
		}
	}
	chyba();
}

int polozPredmet(const char *item, int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return polozPredmetNa(item, robot->x, robot->y, handle);
	}
	chyba();
}

int polozNa(unsigned int x, unsigned int y, int handle)
{
	int stat;

	if(GetRobot(handle) && (x >= 0) && (y >= 0) && (x < world.width) && (y < world.height))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		else SDL_Delay(world.command_delay / 10);
		if(stat = ViewPollEvents()) return stat;

		if(world.map[y * world.width + x].num_marks < 8) world.map[y * world.width + x].num_marks++;
		else chyba();

		printf("KPutAt(%d, %d) after: %d\n", x, y, world.map[y * world.width + x].num_marks);
		ViewUpdateAt(x, y);
		return 1;
	}
	chyba();
}

int poloz(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return polozNa(robot->x, robot->y, handle);
	}
	chyba();
}

int zvedniNa(unsigned int x, unsigned int y, int handle = -1)
{
	int stat;

	if(GetRobot(handle) && (x >= 0) && (y >= 0) && (x < world.width) && (y < world.height))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;
		
		if(world.map[y * world.width + x].num_marks)
		{
			world.map[y * world.width + x].num_marks--;
			printf("KGetAt(%d, %d) after: %d\n", x, y, world.map[y * world.width + x].num_marks);
			ViewUpdateAt(x, y);
			return 1;
		}
	}
	chyba();
}

int zvedni(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return zvedniNa(robot->x, robot->y, handle);
	}
	chyba();
}

int zvedniPredmetNa(unsigned int x, unsigned int y, int handle = -1)
{
	int stat;

	if(GetRobot(handle) && (x >= 0) && (y >= 0) && (x < world.width) && (y < world.height))
	{
		if(!world.fast) SDL_Delay(world.command_delay);
		if(stat = ViewPollEvents()) return stat;
		
		if(world.map[y * world.width + x].item != -1)
		{
			world.map[y * world.width + x].item = -1;
			ViewUpdateAt(x, y);
			return 1;
		}
	}
	chyba();
}

int zvedniPredmet(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return zvedniPredmetNa(robot->x, robot->y, handle);
	}
	chyba();
}

int editujSvet()
{
	int x = 0, y = 0, i;
	SDL_Event event;
	Robot *it;
	int drag = 0;

	if(world_inited != 2)
	{
		fprintf(stderr, "World not created.");
		return -1;
	}

	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
	ViewDrawCursor(x, y);
	while(1)
	{
		SDL_WaitEvent(&event);
		switch(event.type)
		{
		case SDL_KEYDOWN:
			switch(event.key.keysym.sym)
			{
			case SDLK_RETURN:
				ViewUpdateAt(x, y);
				return 1;
			case SDLK_RIGHT:
				if(x < world.width - 1)
				{
					if(drag) it->x++;
					ViewUpdateAt(x, y);
					x++;
					ViewUpdateAt(x, y);
					if(!drag) ViewDrawCursor(x, y);
				}
				break;
			case SDLK_UP:
				if(y > 0)
				{
					if(drag) it->y--;
					ViewUpdateAt(x, y);
					y--;
					ViewUpdateAt(x, y);
					if(!drag) ViewDrawCursor(x, y);
				}
				break;
			case SDLK_LEFT:
				if(x > 0)
				{
					if(drag) it->x--;
					ViewUpdateAt(x, y);
					x--;
					ViewUpdateAt(x, y);
					if(!drag) ViewDrawCursor(x, y);
				}
				break;
			case SDLK_DOWN:
				if(y < world.height - 1)
				{
					if(drag) it->y++;
					ViewUpdateAt(x, y);
					y++;
					ViewUpdateAt(x, y);
					if(!drag) ViewDrawCursor(x, y);
				}
				break;
			case SDLK_0:
			case SDLK_KP0:
				if(drag) break;
				world.map[y * world.width + x].num_marks = 0;
				ViewUpdateAt(x, y);
				ViewDrawCursor(x, y);
				break;
			case SDLK_1:
			case SDLK_KP1:
				if(drag) break;
				world.map[y * world.width + x].num_marks = 1;
				ViewUpdateAt(x, y);
				ViewDrawCursor(x, y);
				break;
			case SDLK_2:
			case SDLK_KP2:
				if(drag)
				{
					it->dir = 3;
					ViewUpdateAt(x, y);
				}
				else
				{
					world.map[y * world.width + x].num_marks = 2;
					ViewUpdateAt(x, y);
					ViewDrawCursor(x, y);
				}
				break;
			case SDLK_3:
			case SDLK_KP3:
				if(drag) break;
				world.map[y * world.width + x].num_marks = 3;
				ViewUpdateAt(x, y);
				ViewDrawCursor(x, y);
				break;
			case SDLK_4:
			case SDLK_KP4:
				if(drag)
				{
					it->dir = 2;
					ViewUpdateAt(x, y);
				}
				else
				{
					world.map[y * world.width + x].num_marks = 4;
					ViewUpdateAt(x, y);
					ViewDrawCursor(x, y);
				}
				break;
			case SDLK_5:
			case SDLK_KP5:
				if(drag) break;
				world.map[y * world.width + x].num_marks = 5;
				ViewUpdateAt(x, y);
				ViewDrawCursor(x, y);
				break;
			case SDLK_6:
			case SDLK_KP6:
				if(drag)
				{
					it->dir = 0;
					ViewUpdateAt(x, y);
				}
				else
				{
					world.map[y * world.width + x].num_marks = 6;
					ViewUpdateAt(x, y);
					ViewDrawCursor(x, y);
				}
				break;
			case SDLK_7:
			case SDLK_KP7:
				if(drag) break;
				world.map[y * world.width + x].num_marks = 7;
				ViewUpdateAt(x, y);
				ViewDrawCursor(x, y);
				break;
			case SDLK_8:
			case SDLK_KP8:
				if(drag)
				{
					it->dir = 1;
					ViewUpdateAt(x, y);
				}
				else
				{
					world.map[y * world.width + x].num_marks = 8;
					ViewUpdateAt(x, y);
					ViewDrawCursor(x, y);
				}
				break;
			case SDLK_F12:
				// F12 vyma¾e mìsto a vrátí ho do výchozího stavu. Pozor, zatím bez dotazù!
				for (i = 0; i<(world.width * world.height); i++)
				{
					world.map[i].num_marks = 0;
					world.map[i].wall = 0;
					world.map[i].item = -1;
				}
				if(drag) drag=0;	// pustit Karla a umístit na start (v¹echny roboty)
				x = 0;
				y = world.height - 1;
				for(it = world.robots; it != NULL; it = it->next)
				{
					it->x = x;
					it->y = y;
					y--;
					if (y < 0)
					{
						x++;
						y = world.height -1;
						if (x == world.width)
						{
							printf ("DEBUG: Too much robots, all robots cannot be in this very small world!");
							break;
						}
					}

				}
				// pøekreslit v¹e a umístit kurzor
				for (x = 0; x < world.width; x++)
					for (y = 0; y < world.height; y++)
						ViewUpdateAt(x, y);
				x = 0;
				y = 0;
				ViewDrawCursor(x, y);
				break;
			case SDLK_SPACE:
				if(!drag)
				{
					for(it = world.robots; it != NULL; it = it->next)
					{
						if((it->x == x) && (it->y == y)) break;
					}
					if(it)
					{
						drag = 1;
						ViewUpdateAt(x, y);
						break;
					}
					else
					{
						world.map[y * world.width + x].wall = !world.map[y * world.width + x].wall;
						ViewUpdateAt(x, y);
						ViewDrawCursor(x, y);
						break;
					}
				}
				else
				{
					drag = 0;
					ViewDrawCursor(x, y);
				}
			}
			break;
		case SDL_QUIT:
			SDL_Quit();
			exit(0);
		}
	}
	SDL_EnableKeyRepeat(0, SDL_DEFAULT_REPEAT_INTERVAL);
}

int cekej()
{
	SDL_Event event;

	if(world_inited != 2)
	{
		fprintf(stderr, "World not created.");
		return -1;
	}
	while(1)
	{
		SDL_WaitEvent(&event);
		switch(event.type)
		{
		case SDL_KEYDOWN:
		case SDL_QUIT:
			SDL_Quit();
			exit(0);

		}
	}
}

int jeVychod(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return (robot->dir == 0) ? 1 : 0;
	}
	chyba();
}

int jeSever(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return (robot->dir == 1) ? 1 : 0;
	}
	chyba();
}

int jeZapad(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return (robot->dir == 2) ? 1 : 0;
	}
	chyba();
}

int jeJih(int handle)
{
	Robot *robot;

	if(robot = GetRobot(handle))
	{
		return (robot->dir == 3) ? 1 : 0;
	}
	chyba();
}

int jeNahoda()
{
	return rand() & 1;
}

int jeStisknuto()
{
	Robot		*robot;
	SDL_Event	event;
	int		result,i;
	Uint8		*keystate;

	result = 0;

	SDL_PumpEvents();
	keystate = SDL_GetKeyState(NULL);
	for (i = 0; i < 128; i++)
	{
		if (keystate[i]) result = 1;
	}
	return result;
}

int jePlno(int handle)
{
	Robot	*robot;
	int	marks, result;

	if(robot = GetRobot(handle))
	{
		result = 0;
		marks = world.map[robot->y * world.width + robot->x].num_marks;
		if (marks >= 8) result = 1;
		return (result);
	}
	return -1;
}

int chyba()
{
	SDL_Event event;

	if(world_inited != 2)
	{
		fprintf(stderr, "World not created.");
		return -1;
	}
	for(int i = 0; i < world.width * world.grid; i+=2)
	{
		SDL_Rect rect;
		rect.x = i; rect.y = 0; rect.h = world.height * world.grid; rect.w = 1;
		SDL_FillRect(world.screen, &rect, SDL_MapRGB(world.screen->format, 255, 0, 0));
		SDL_UpdateRect(world.screen, i, 0, 1, world.height * world.grid);
		SDL_Delay(2);
		while(SDL_PollEvent(&event))
		{
			switch(event.type)
			{
			case SDL_KEYDOWN:
			case SDL_QUIT:
				SDL_Quit();
				exit(0);
			}
		}
	}

	konec();
}
