/*
 *   Karel v Ccku
 *
 *   Copyright (c) 2012 Tomas Blaha
 *
 *   Permission is hereby granted, free of charge, to any person
 *   obtaining a copy of this software and associated documentation
 *   files (the "Software"), to deal in the Software without
 *   restriction, including without limitation the rights to use,
 *   copy, modify, merge, publish, distribute, sublicense, and/or sell
 *   copies of the Software, and to permit persons to whom the
 *   Software is furnished to do so, subject to the following
 *   conditions:
 *
 *   The above copyright notice and this permission notice shall be
 *   included in all copies or substantial portions of the Software.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *   OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *   NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *   OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef __KAREL_H
#define __KAREL_H

#include <SDL/SDL.h>

int GetNumErrors();
char *GetError(int n);

////////////////////////////////////////////////////////////
// World management
int KInit();
int KCreateWorld(unsigned int width, unsigned int height);
//int KConnectWorld(char *hostname, int port);
int KSaveWorld(const char *fname);
int KLoadWorld(const char *fname);
int KEditWorld();
int konec();

void priprav(const int w, const int h);
void priprav(const char* uloha);
float nahoda(int minimum, int maximum);

////////////////////////////////////////////////////////////
// Robots management
int KCreateRobot();
int KCreateRobotAt(unsigned int x, unsigned int y, unsigned int dir);
//int DestroyRobot(int handle = -1);

////////////////////////////////////////////////////////////
// Basic Karel commands
int vlevo(int handle = -1);
int vpravo(int handle = -1);
int krok(int handle = -1);
int poloz(int handle = -1);
int polozPredmet(const char *item, int handle = -1);
int zvedni(int handle = -1);
int zvedniPredmet(int handle = -1);
int jeZed(int handle = -1);
int jePredmet(int handle = -1);
int jeZnacka(int handle = -1);
int jeRobot(int handle = -1);
int jeVychod(int handle = -1);
int jeSever(int handle = -1);
int jeZapad(int handle = -1);
int jeJih(int handle = -1);
int jeNahoda();
int jeZmacknuto();
int jePlno(int handle = -1);

////////////////////////////////////////////////////////////
// Extended Karel commands
int polozNa(unsigned int x, unsigned int y, int handle = -1);
int polozPredmetNa(const char *item, unsigned int x, unsigned int y, int handle = -1);
int jeZedNa(unsigned int x, unsigned int y, int handle = -1);
int jeZnackaNa(unsigned int x, unsigned int y, int handle = -1);
int jePredmetNa(unsigned int x, unsigned int y, int handle = -1);
int jeRobotNa(unsigned int x, unsigned int y, int handle = -1);
int cekej();
int chyba();

typedef struct tagRobot
{
	int handle;
	int x, y, dir;
	struct tagRobot *next;
} Robot;

typedef struct
{
	int wall, item, num_marks;
} WorldField;

typedef struct
{
	int width, height, grid;
	WorldField *map;
	SDL_Surface *screen, *data, *items;
	char item_names[256][32];
	int num_items;
	int view_type, view_offset_x, view_offset_y;
	int command_delay;
	Robot *robots;
	int autoincrement;
	int fast;
} World;

#endif
